unit MainForm;

{$IMPORTEDDATA OFF}

interface

uses NForm, NCommand, NConf, NGraphics, Player, Playlist, InfoDialog;

var
  Config: TConf;
  ProfDir: WideString;

type
  TSeekBar = class(TObject)
    Canvas: TMemCanvas;
    Width: Integer;
    Height: Integer;
    LMsg: string;
    RMsg: string;
    Pos: Integer;
    SliderVisible: Boolean;
    Font: TFont;
    constructor Create(const Parent: TForm);
    destructor Destroy; override;
    procedure Draw(const Dist: TCanvas; X, Y: Integer);
  end;

  TAspectMode = (amAuto, amOrg, am43, am169);

  TNkVpMainForm = class(TForm)
    AspectMode: TAspectMode;
    Command: TCommand;
    CursorTimer: TTimer;
    Dim: Boolean;
    Dim2: Boolean;
    ExitAtEnd: Boolean;
    FitMovie: Boolean;
    InfoDialog: TInfoDialog;
    Mute: Boolean;
    OldCursorPt: TPoint;
    Player: TPlayer;
    Playlist: TPlaylist;
    RepeatPlay: Boolean;
    RestCursorCount: Integer;
    SavePlayerState: TPlayerState;
    SeekBar: TSeekBar;
    SeekBarTimer: TTimer;
    ShowFormatInfo: Boolean;
    ShowSeekBar: Boolean;
    constructor Create; override;
    destructor Destroy; override;
    procedure Closing; override;
    procedure DispatchCommand(Id: Integer); override;
    procedure DropFiles(const ListDropFiles: TListDropFiles); override;
    procedure MouseDoubleClick(const Button: TMouseButton; CX, CY: Integer);
        override;
    procedure MouseDown(const Button: TMouseButton; CX, CY: Integer); override;
    procedure MouseMove(CX, CY: Integer); override;
    procedure MouseUp(const Button: TMouseButton; CX, CY: Integer); override;
    procedure MouseWheel(Delta: Integer); override;
    procedure Paint(const Canvas: TCanvas; Left, Top, Width, Height: Integer);
        override;
    procedure RecvFromRemote(const S: WideString); override;
    procedure Resize; override;
    procedure ShowContextMenu(SX, SY: Integer); override;

    function CalcVideoWidth: Integer;
    procedure CheckCursorPos;
    procedure LoadFromPlaylist(Index: Integer; ResetClientSize: Boolean);
    procedure UpdatePlayerBounds;
    procedure UpdateSeekBar;
    procedure UpdateTitle;
    procedure UpdateVolume;
    procedure Zoom(Value: Integer);
  published
    procedure cmAbout;
    procedure cmAngle;
    procedure cmAspect169;
    procedure cmAspect43;
    procedure cmAspectAuto;
    procedure cmAspectOrg;
    procedure cmAudioStream;
    //procedure cmClosedCaption;
    procedure cmDim;
    procedure cmDim2;
    procedure cmExit;
    procedure cmFilterMenu;
    procedure cmFitMovie;
    procedure cmFullScreen;
    procedure cmInfoWindow;
    procedure cmMute;
    procedure cmPause;
    procedure cmPlaylist;
    procedure cmRepeat;
    procedure cmSaveScreenCapture;
    procedure cmSeekBack;
    procedure cmSeekBackFast;
    procedure cmSeekBackFaster;
    procedure cmSeekForward;
    procedure cmSeekForwardFast;
    procedure cmSeekForwardFaster;
    procedure cmShowFormatInfo;
    procedure cmSeekOnKeyframe;
    procedure cmShowSeekBar;
    procedure cmShowSubpicture;
    procedure cmSkipNext;
    procedure cmSkipNextTitle;
    procedure cmSkipPrev;
    procedure cmSkipPrevTitle;
    procedure cmStayOnTop;
    procedure cmStop;
    procedure cmSubpictureStream;
    procedure cmTitle;
    procedure cmZoom100;
    procedure cmZoom150;
    procedure cmZoom200;
    procedure cmZoom300;
    procedure cmZoom400;
    procedure cmZoom50;
  end;

implementation

uses NTypes, NSys, NLib, MediaFormat, Menu, AboutDialog;

const
  FirstDvdAudioMenuId = 1000;
  LastDvdAudioMenuId = FirstDvdAudioMenuId + 100;
  FirstDvdSubpictureMenuId = 1200;
  LastDvdSubpictureMenuId = FirstDvdSubpictureMenuId + 100;
  FirstDvdAngleMenuId = 1350;
  LastDvdAngleMenuId = FirstDvdAngleMenuId + 10;
  FirstDvdTitleMenuId = 1400;
  LastDvdTitleMenuId = FirstDvdTitleMenuId + 100;
  FirstFilterMenuId = 3000;
  LastFilterMenuId = FirstFilterMenuId + 500;
  FirstPlaylistId = 5000;
  LastPlaylistId = FirstPlaylistId + 10000;

function EscapeAmp(const S: WideString): WideString;
var
  i: Integer;
begin
  Result := '';
  for i := 1 to Length(S) do
    if S[i] = '&' then
      Result := Result + '&&'
    else
      Result := Result + S[i];
end;


{ TSeekBar }

constructor TSeekBar.Create;
begin
  Canvas := TMemCanvas.Create(Parent.Handle);
  Font := TFont.Create(lfStatus);
end;

destructor TSeekBar.Destroy;
begin
  Font.Free;
  Canvas.Free;
end;

procedure TSeekBar.Draw(const Dist: TCanvas; X, Y: Integer);
var
  ty: Integer;
begin
  Canvas.SetSize(Width, Height);
  Canvas.BeginDraw;

  // background
  Canvas.FillRect(0, 0, Width, Height, $333333);
  Canvas.FillRect(0, 1, Width, Height, $000000);
  // slider
  if SliderVisible then
  begin
    Canvas.FillRect(Pos-2, 2, 5, Height-4, $555555);
    Canvas.FillRect(Pos,   2, 1, Height-4, $000000);
  end;
  // text
  Canvas.SetFont(Font);
  ty := (Height - 1 - Canvas.FontHeight) div 2 + 1;
  Canvas.SetTextColor($777777);
  Canvas.SetTextAlign(taLeft);
  Canvas.DrawText(0, ty, LMsg);
  Canvas.SetTextAlign(taRight);
  Canvas.DrawText(Width, ty, RMsg);

  Canvas.Draw(Dist, X, Y);
  Canvas.EndDraw;
end;


{ TNkVpMainForm }

constructor TNkVpMainForm.Create;
begin
  inherited;

  Command := TCommand.Create(Self, [
    cmAbout,
    cmAngle,
    cmAspect169,
    cmAspect43,
    cmAspectAuto,
    cmAspectOrg,
    cmAudioStream,
    cmDim,
    cmDim2,
    cmExit,
    cmFilterMenu,
    cmFitMovie,
    cmFullScreen,
    cmInfoWindow,
    cmMute,
    cmPause,
    cmPlaylist,
    cmRepeat,
    cmSaveScreenCapture,
    cmSeekBack,
    cmSeekBackFast,
    cmSeekBackFaster,
    cmSeekForward,
    cmSeekForwardFast,
    cmSeekForwardFaster,
    cmSeekOnKeyframe,
    cmShowFormatInfo,
    cmShowSeekBar,
    cmShowSubpicture,
    cmSkipNext,
    cmSkipNextTitle,
    cmSkipPrev,
    cmSkipPrevTitle,
    cmStayOnTop,
    cmStop,
    cmSubpictureStream,
    cmTitle,
    cmZoom100,
    cmZoom150,
    cmZoom200,
    cmZoom300,
    cmZoom400,
    cmZoom50
  ]);

  SetBackground($000000);
  SetText(App.Name);

  Player := TPlayer.Create(Handle);
  SeekBar := TSeekBar.Create(Self);
  SeekBar.Height := 13;
  Playlist := TPlaylist.Create;
  CursorTimer := TTimer.Create(Self, CheckCursorPos, 500);
  SeekBarTimer := TTimer.Create(Self, UpdateSeekBar, 100);

  InfoDialog := TInfoDialog.Create;
  InfoDialog.Player := Player;

  EnableDropFiles(True);

  LoadFormPos(Self, Config);
  if Config.GetStr('renderer') = 'vmr9' then
    Player.Renderer := prVMR9;
  if Config.GetStr('renderer') = 'evr' then
    Player.Renderer := prEVR;
  FitMovie := Config.GetBool('fitmovie');
  Player.SeekOnKeyframe := Config.GetBool('seekonkeyframe');

  ShowSeekBar := True;
end;

destructor TNkVpMainForm.Destroy;
begin
  InfoDialog.Free;
  SeekBarTimer.Free;
  CursorTimer.Free;
  Playlist.Free;
  SeekBar.Free;
  Player.Free;
  Command.Free;
end;

procedure TNkVpMainForm.Paint(const Canvas: TCanvas;
    Left, Top, Width, Height: Integer);
var
  cw, ch: Integer;
begin
  if ShowSeekBar then
  begin
    GetClientSize(cw, ch);
    SeekBar.Width := cw;
    SeekBar.Draw(Canvas, 0, ch-SeekBar.Height);
  end;
end;

procedure TNkVpMainForm.Closing;
var
  position: TConf;
begin
  if not Config.GetBool('saveposition') then
    Exit;

  position := TConf.Create;
  try
    SaveFormPos(Self, position);
    try
      position.SaveToFile(JoinPath([ProfDir, 'position']));
    except
      on E: EOSError do App.Error(E.Path + LineSep + LineSep + E.Msg);
    end;
  finally
    position.Free;
  end;
end;

procedure TNkVpMainForm.LoadFromPlaylist;
var
  //wnd: HWnd;
  dir: TDir;
  path: WideString;
begin
  if (Index < 0) or (Index >= Playlist.Count) then
    Exit;

  path := Playlist.Items[Index];

  try
    ClearDir(dir);
    if IsUrl(path) then
      SetText('Connecting... - ' + App.Name)
    else
      dir := DirStat(path);
    Player.LoadFile(path);
  except
    on E: EOSError do
    begin
      Error(path + LineSep + LineSep + E.Msg);
      Exit;
    end;
    on E: EDShowError do
    begin
      Error(path + LineSep + LineSep + E.Msg);
      UpdateTitle;
      SeekBar.LMsg := '';
      SeekBar.RMsg := '';
      SeekBar.Pos := 0;
      Invalidate;
      Exit;
    end;
  end;

  if Player.WarnMsg <> '' then
    Warn(path + LineSep + LineSep + Player.WarnMsg);

  Playlist.Cur := Index;

  {
  wnd := GetWindow(Handle, GW_CHILD);
  if IsWindow(wnd) then
    InvalidateRect(wnd, nil, False);
  }

  if ResetClientSize then
  begin
    cmAspectAuto;
    cmZoom100;
  end else
    UpdatePlayerBounds;

  UpdateVolume;
  Player.Play;
  SeekBarTimer.Start;

  InfoDialog.MTime := dir.MTime;
  InfoDialog.Size := dir.Size;
  if InfoDialog.Visible then
    InfoDialog.UpdateInfo;
end;

procedure TNkVpMainForm.UpdateTitle;
var
  title, path: WideString;
begin
  if not Player.Loaded then
  begin
    SetText(App.Name);
    Exit;
  end;

  if Player.SourceType = pstDvd then
    title := '<DVD> ' + AbsPath(Player.Path)
  else begin
    if Player.SourceType = pstURL then
      path := Player.Path
    else
      path := BaseName(Player.Path);
    if Player.HasVideo then
      title := path + Format(' ({}x{} {}fps) {}%', [
          Player.MediaInfo.VideoWidth,
          Player.MediaInfo.VideoHeight,
          FStr(Player.MediaInfo.VideoFrameRate, 3),
          Player.Height*100 div Player.VideoSize.cy])
    else
      title := path
  end;
  SetText(title + ' - ' + App.Name);
end;

function TNkVpMainForm.CalcVideoWidth: Integer;
begin
  with Player do
    if HasVideo then
      case AspectMode of
        am169:
          Result := Trunc(VideoSize.cy * 16/9 + 0.5);
        am43:
          Result := Trunc(VideoSize.cy * 4/3 + 0.5);
        amAuto:
          if VideoAspect.cy > 0 then
            Result := Trunc(VideoSize.cy * (VideoAspect.cx / VideoAspect.cy) + 0.5)
          else
            Result := VideoSize.cx;
        amOrg:
          Result := VideoSize.cx;
      else
        Result := VideoSize.cx;
      end
    else
      Result := 320;
end;

procedure TNkVpMainForm.UpdatePlayerBounds;
var
  w, h, cw, ch, vw, vh: Integer;
begin
  GetClientSize(cw, ch);
  if ShowSeekBar then
    Dec(ch, SeekBar.Height);
  w := cw;
  h := ch;
  vw := CalcVideoWidth;
  vh := Player.VideoSize.cy;
  if (vw <> 0) and (vh <> 0) then
  begin
    if vw*ch > vh*cw then
      h := (vh * cw) div vw
    else if vw*ch < vh*cw then
      w := (vw * ch) div vh;
    Player.SetBounds((cw-w) div 2, (ch-h) div 2, w, h);
  end;
  UpdateTitle;
end;

procedure TNkVpMainForm.Zoom;
var
  w, h: Integer;
begin
  if FullScreen then
  begin
    SetFullScreen(False, False);
    CursorTimer.Stop;
    Cursor.Show;
  end;
  Restore;
  w := CalcVideoWidth * Value div 100;
  h := Player.VideoSize.cy * Value div 100 + SeekBar.Height;
  if Player.Loaded and not Player.HasVideo then
    Dec(h);
  SetClientSize(w, h);
end;

procedure TNkVpMainForm.UpdateSeekBar;
var
  cw, ch, pos: Integer;
  pstat, pindex, info: string;
begin
  GetClientSize(cw, ch);

  pstat := '';
  if RepeatPlay then
    pstat := pstat + ' REPEAT';
  if Mute then
    pstat := pstat + ' MUTE';
  if Dim then
    pstat := pstat + ' DIM';
  if Dim2 then
    pstat := pstat + ' DIM2';

  if not Player.Loaded then
  begin
    SeekBar.LMsg := pstat;
    SeekBar.RMsg := '';
    Invalidate(0, ch-SeekBar.Height, cw, SeekBar.Height, False);
    Exit;
  end;

  pos := Player.Pos;
  if (Player.Len > 0) and (pos >= Player.Len) then
    if RepeatPlay then
      Player.SeekTo(0)
    else if ExitAtEnd then
      Close
    else begin
      Player.Stop;
      SeekBarTimer.Stop;
      cmSkipNext;
    end;

  if Player.Len > 0 then
  begin
    SeekBar.SliderVisible := True;
    SeekBar.Pos := Trunc(pos / Player.Len * cw)
  end else
    SeekBar.SliderVisible := False;

  if Playlist.Count > 1 then
    pindex := Format(' {}/{} ', [Playlist.Cur+1, Playlist.Count])
  else if Player.Sourcetype = pstDvd then
    pindex := Format(' {}-{}/{} ',
        [Player.DvdTitle, Player.DvdChapter, Player.DvdChapterCount])
  else
    pindex := '';

  case Player.State of
    psStopped: pstat := ' STOP' + pstat;
    psPaused: pstat := ' PAUSE' + pstat;
  end;

  info := '';
  if ShowFormatInfo then
    with Player.MediaInfo do begin
      info := AudioCodec;
      if AudioSampleRate > 0 then
        info := info + ' ' + IStr(AudioSampleRate div 1000) + 'kHz';
      if AudioBitrate > 0 then
        info := info + ' ' + IStr(AudioBitRate div 1000) + 'k';
      case AudioChannels of
        0, 1: ;
        2: info := info + ' st';
      else
        info := info + ' ' + IStr(AudioChannels) + 'ch';
      end;
      if VideoCodec <> '' then
        info := info + ' ' + VideoCodec;
    end;
  info := info + ' ' + FormatTime(pos) + ' ';
  // round off the media length
  if Player.Len > -1 then
    info := info + '/ ' + FormatTime(Player.Len+500) + ' ';

  SeekBar.LMsg := pindex + pstat;
  SeekBar.RMsg := info;
  Invalidate(0, ch-SeekBar.Height, cw, SeekBar.Height, False);
end;

procedure TnkVpMainForm.UpdateVolume;
var
  vol: Integer;
begin
  if Mute then
    vol := -10000
  else if Dim then
    vol := -2000
  else if Dim2 then
    vol := -1000
  else
    vol := 0;
  Player.SetVolume(vol);
end;

procedure TNkVpMainForm.CheckCursorPos;
var
  pt: TPoint;
begin
  pt := Cursor.GetPosition;
  if (pt.x = OldCursorPt.x) and (pt.y = OldCursorPt.y) then
  begin
    Inc(RestCursorCount);
    if RestCursorCount = 3000 div CursorTimer.Interval then
      Cursor.Hide;
  end
  else begin
    RestCursorCount := 0;
    Cursor.Show;
  end;
  OldCursorPt := pt;
end;

procedure TNkVpMainForm.DispatchCommand(Id: Integer);
begin
  case Id of
    FirstDvdAudioMenuId .. LastDvdAudioMenuId:
      Player.DvdSetAudio(Id - FirstDvdAudioMenuId);
    FirstDvdSubpictureMenuId .. LastDvdSubpictureMenuId:
      Player.DvdSetSubpicture(Id - FirstDvdSubpictureMenuId);
    FirstDvdAngleMenuId .. LastDvdAngleMenuId:
      Player.DvdSetAngle(Id - FirstDvdAngleMenuId + 1);
    FirstDvdTitleMenuId .. LastDvdTitleMenuId:
      begin
        Player.DvdPlayTitle(Id - FirstDvdTitleMenuId);
        UpdatePlayerBounds; // apply VideoAspect
      end;
    FirstFilterMenuId .. LastFilterMenuId:
      try
        Player.ShowFilterProperty(Id - FirstFilterMenuId);
      except
        on E: EDShowError do Error(E.Msg);
      end;
    FirstPlaylistId .. LastPlaylistId:
      LoadFromPlaylist(Id - FirstPlaylistId, False);
  else
    Command.DispatchCommand(Id);
    UpdateSeekBar;
  end;
end;

procedure TNkVpMainForm.ShowContextMenu(SX, SY: Integer);
var
  i, j, id: Integer;
  n, cur: Cardinal;
  display: Boolean;
begin
  with TPopupMenu.Create do
    try
      Command.AddMenuItems(Handle, MenuItems);
      if Player.SourceType = pstDvd then
      begin
        DeleteItemByPos(0); // Delete Playlist Menu

        id := Command.Id(cmTitle);
        for i := 1 to Player.DvdTitleCount do
          InsertItem(id, FirstDvdTitleMenuId+i, IStr(i));
        CheckItem(FirstDvdTitleMenuId+Player.DvdTitle, True);
        DeleteItem(id); // delete (none)

        id := Command.Id(cmAudioStream);
        Player.DvdGetAudio(n, cur);
        for i := 0 to n-1 do
          if Player.DvdIsAudioEnabled(i) then
            InsertItem(id, FirstDvdAudioMenuId+i, Player.DvdGetAudioLanguage(i));
        CheckItem(FirstDvdAudioMenuId+cur, True);
        DeleteItem(id); // delete (none)

        id := Command.Id(cmSubpictureStream);
        Player.DvdGetSubpicture(n, cur, display);
        j := 0;
        for i := 0 to n-1 do
          if Player.DvdIsSubpictureEnabled(i) then
          begin
            InsertItem(id, FirstDvdSubpictureMenuId+i, Player.DvdGetSubpictureLanguage(i));
            Inc(j);
          end;
        if j = 0 then
          GrayItemByPos(2)
        else begin
          CheckItem(FirstDvdSubpictureMenuId+cur, True);
          CheckItem(Command.Id(cmShowSubpicture), display);
          DeleteItem(id); // delete (none)
        end;

        id := Command.Id(cmAngle);
        Player.DvdGetAngle(n, cur);
        for i := 0 to n-1 do
          InsertItem(id, FirstDvdAngleMenuId+i, IStr(i+1));
        CheckItem(FirstDvdAngleMenuId+cur-1, True);
        DeleteItem(id); // delete (none)

        //id := Command.Id(cmClosedCaption);
        //CheckItem(id, Player.ClosedCaption);
      end else
      begin
        DeleteItemByPos(1); // Title List Menu
        DeleteItemByPos(1); // Audio Stream Menu
        DeleteItemByPos(1); // Subpicture Stream Menu
        DeleteItemByPos(1); // Angle
        //DeleteItemByPos(1); // Closed Caption Menu
      end;

      if Player.Loaded then
      begin
        if Player.State = psStopped then
          GrayItem(Command.Id(cmStop));

        if Player.SourceType = pstDvd then begin
          if Player.DvdChapter = 1 then
            GrayItem(Command.Id(cmSkipPrev));
          if Player.DvdChapter = Player.DvdChapterCount then
            GrayItem(Command.Id(cmSkipNext));
        end else begin
          id := Command.Id(cmPlaylist);
          for i := 0 to Playlist.Count-1 do
            InsertItem(id, FirstPlaylistId+i, EscapeAmp(BaseName(Playlist.Items[i])));
          DeleteItem(id);
          CheckItem(FirstPlaylistId+Playlist.Cur, True);

          if Playlist.Cur = 0 then
            GrayItem(Command.Id(cmSkipPrev));
          if Playlist.Cur = Playlist.Count - 1 then
            GrayItem(Command.Id(cmSkipNext));
        end;

        id := Command.Id(cmFilterMenu);
        for i := 0 to High(Player.FilterInfo) do
          with Player.filterInfo[i] do
          begin
            InsertItem(id, FirstFilterMenuId+i, Name);
            if Prop = nil then
              GrayItem(FirstFilterMenuId+i);
          end;
        DeleteItem(id);

        if not Player.HasVideo then
          GrayItem(Command.Id(cmSaveScreenCapture));
      end else
      begin
        GrayItem(Command.Id(cmPlaylist));
        GrayItem(Command.Id(cmPause));
        GrayItem(Command.Id(cmStop));
        GrayItem(Command.Id(cmSkipNext));
        GrayItem(Command.Id(cmSkipPrev));
        GrayItem(Command.Id(cmZoom50));
        GrayItem(Command.Id(cmZoom100));
        GrayItem(Command.Id(cmZoom150));
        GrayItem(Command.Id(cmZoom200));
        GrayItem(Command.Id(cmZoom300));
        GrayItem(Command.Id(cmZoom400));
        GrayItem(Command.Id(cmSaveScreenCapture));
        GrayItem(Command.Id(cmFilterMenu));
        GrayItem(Command.Id(cmInfoWindow));
      end;

      CheckItem(Command.Id(cmFullScreen), FullScreen);
      CheckItem(Command.Id(cmStayOnTop), StayOnTop);
      CheckItem(Command.Id(cmShowSeekBar), ShowSeekBar);
      CheckItem(Command.Id(cmMute), Mute);
      CheckItem(Command.Id(cmRepeat), RepeatPlay);
      CheckItem(Command.Id(cmDim), Dim);
      CheckItem(Command.Id(cmShowFormatInfo), ShowFormatInfo);
      CheckItem(Command.Id(cmFitMovie), FitMovie);
      CheckItem(Command.Id(cmSeekOnKeyframe), Player.SeekOnKeyframe);
      if Length(Player.Keyframes) = 0 then
        GrayItem(Command.Id(cmSeekOnKeyframe));

      Popup(Self.Handle, SX, SY)
    finally
      Free;
    end;
end;

procedure TNkVpMainForm.DropFiles(const ListDropFiles: TListDropFiles);
var
  path: WideString;
begin
  Playlist.Clear;
  while ListDropFiles.Each(path) do
    Playlist.Add(path);
  Playlist.Sort;
  LoadFromPlaylist(0, not FitMovie);
end;

procedure TNkVpMainForm.RecvFromRemote(const S: WideString);
var
  args: TWideStringArray;
  i: Integer;
begin
  args := SplitCommandLineW(S);
  Playlist.Clear;
  for i := 0 to High(args) do
    Playlist.Add(args[i]);
  Restore;
  LoadFromPlaylist(0, not FitMovie);
end;

procedure TNkVpMainForm.MouseDown(const Button: TMouseButton; CX, CY: Integer);
var
  cw, ch: Integer;
begin
  if Button <> mbLeft then
    Exit;

  GetClientSize(cw, ch);
  if CY >= (ch - SeekBar.Height) then
  begin
    if (Player.Len <= 0) or not Player.Seekable then
      Exit;
    SetCapture(True);
    SavePlayerState := Player.State;
    if SavePlayerState = psPlaying then
    begin
      Player.Pause;
      SeekBarTimer.Stop;
    end;
    Player.SeekTo(Trunc(CX / cw * Player.Len));
  end else
    cmPause;
  UpdateSeekBar;
end;

procedure TNkVpMainForm.MouseUp(const Button: TMouseButton; CX, CY: Integer);
begin
  if Button <> mbLeft then
    Exit;

  if Capture then
  begin
    SetCapture(False);
    if SavePlayerState = psPlaying then
    begin
      Player.Play;
      SeekBarTimer.Start;
    end;
  end;
  UpdateSeekBar;
end;

// player window sends a double click message
procedure TNkVpMainForm.MouseDoubleClick(const Button: TMouseButton;
    CX, CY: Integer);
begin
  MouseDown(Button, CX, CY);
end;

procedure TNkVpMainForm.MouseMove(CX, CY: Integer);
var
  cw, ch: Integer;
begin
  if Capture then
  begin
    GetClientSize(cw, ch);
    Player.SeekTo(Trunc(CX / cw * Player.Len));
    UpdateSeekBar;
  end;
end;

procedure TNkVpMainForm.Resize;
begin
  UpdatePlayerBounds;
  Invalidate;
end;

procedure TNkVpMainForm.MouseWheel(Delta: Integer);
begin
  if Delta > 0 then
  begin
    if mkShift in ModifierKeys then
      cmSeekBackFast
    else
      cmSeekBack;
  end
  else begin
    if mkShift in ModifierKeys then
      cmSeekForwardFast
    else
      cmSeekForward;
  end;
  UpdateSeekBar;
end;


{ Command Methods }

procedure TNkVpMainForm.cmAbout;
begin
  with TAboutDialog.Create do
    try
      ProfDir := MainForm.ProfDir;
      ShowModal(Self);
    finally
      Free;
    end;
end;

procedure TNkVpMainForm.cmAspect169;
begin
  AspectMode := am169;
  UpdatePlayerBounds;
end;

procedure TNkVpMainForm.cmAspect43;
begin
  AspectMode := am43;
  UpdatePlayerBounds;
end;

procedure TNkVpMainForm.cmAspectAuto;
begin
  AspectMode := amAuto;
  UpdatePlayerBounds;
end;

procedure TNkVpMainForm.cmAspectOrg;
begin
  AspectMode := amOrg;
  UpdatePlayerBounds;
end;

procedure TNkVpMainForm.cmExit;
begin
  Close;
end;

procedure TNkVpMainForm.cmFullScreen;
begin
  SetFullScreen(not FullScreen, True);
  if FullScreen then
  begin
    RestCursorCount := 0;
    CursorTimer.Start;
  end
  else begin
    CursorTimer.Stop;
    Cursor.Show;
  end;
end; 

procedure TNkVpMainForm.cmInfoWindow;
begin
  if Player.Loaded and not InfoDialog.Visible then
  begin
    InfoDialog.ShowModeless(Self);
    InfoDialog.UpdateInfo;
  end;
end;

procedure TNkVpMainForm.cmSaveScreenCapture;
var
  fname: WideString;
  bmpbuf: PChar;
  bmpbufsize: Cardinal;
  filterindex: Integer;
begin
  if not Player.HasVideo then
    Exit;
  bmpbuf := nil;
  try
    try
      bmpbufsize := Player.Capture(bmpbuf);
      fname := 'nkvpcapt.bmp';
      if ShowSaveDlg(Handle, ['All Files', '*.*'], ['.bmp'], fname,
          filterindex) then
        with TFile.Create(fname) do
          try
            Write(bmpbuf^, bmpbufsize);
          finally
            Free;
          end;
    except
      on E: EDShowError do Error(E.Msg);
    end;
  finally
    FreeMem(bmpbuf);
  end;
end;

procedure TNkVpMainForm.cmStayOnTop;
begin
  SetStayOnTop(not StayOnTop);
end;

procedure TNkVpMainForm.cmRepeat;
begin
  RepeatPlay := not RepeatPlay;
end;

procedure TNkVpMainForm.cmDim;
begin
  Dim := not Dim;
  Dim2 := False;
  UpdateVolume;
end;

procedure TNkVpMainForm.cmDim2;
begin
  Dim2 := not Dim2;
  Dim := False;
  UpdateVolume;
end;

procedure TNkVpMainForm.cmMute;
begin
  Mute := not Mute;
  UpdateVolume;
end;

procedure TNkVpMainForm.cmPause;
begin
  if Player.State = psPlaying then
  begin
    Player.Pause;
    SeekBarTimer.Stop;
  end
  else begin
    Player.Play;
    SeekBarTimer.Start;
  end;
end;

procedure TNkVpMainForm.cmStop;
begin
  Player.Stop;
  SeekBarTimer.Stop;
  Player.SeekTo(0);
end;

procedure TNkVpMainForm.cmShowSeekBar;
var
  w, h: Integer;
begin
  ShowSeekBar := not ShowSeekBar;
  GetClientSize(w, h);
  if FullScreen or Maximized then
  begin
    Invalidate(0, h-SeekBar.Height, w, SeekBar.Height, True);
    UpdatePlayerBounds
  end
  else begin
    if Player.Loaded and not Player.HasVideo then
      Dec(h);
    if ShowSeekBar then
      SetClientSize(w, h+SeekBar.Height)
    else
      SetClientSize(w, h-SeekBar.Height);
  end;
end;

procedure TNkVpMainForm.cmShowFormatInfo;
begin
  ShowFormatInfo := not ShowFormatInfo;
end;

procedure TNkVpMainForm.cmSeekForward;
begin
  Player.Seek(1000);
end;

procedure TNkVpMainForm.cmSeekBack;
begin
  Player.Seek(-1000);
end;

procedure TNkVpMainForm.cmSeekForwardFast;
begin
  Player.Seek(10000);
end;

procedure TNkVpMainForm.cmSeekBackFast;
begin
  Player.Seek(-10000);
end;

procedure TNkVpMainForm.cmSeekForwardFaster;
begin
  Player.Seek(60000);
end;

procedure TNkVpMainForm.cmSeekBackFaster;
begin
  Player.Seek(-60000);
end;

procedure TNkVpMainForm.cmSeekOnKeyframe;
begin
  Player.SeekOnKeyframe := not Player.SeekOnKeyframe;
end;

procedure TNkVpMainForm.cmSkipNext;
begin
  if Player.SourceType = pstDvd then
    Player.DvdPlayChapter(Player.DvdChapter+1)
  else
    LoadFromPlaylist(Playlist.Cur+1, False);
end;

procedure TNkVpMainForm.cmSkipNextTitle;
begin
  Player.DvdPlayTitle(Player.DvdTitle+1);
  UpdatePlayerBounds; // apply VideoAspect
end;

procedure TNkVpMainForm.cmSkipPrev;
begin
  if Player.SourceType = pstDvd then
    Player.DvdPlayChapter(Player.DvdChapter-1)
  else
    LoadFromPlaylist(Playlist.Cur-1, False);
end;

procedure TNkVpMainForm.cmSkipPrevTitle;
begin
  Player.DvdPlayTitle(Player.DvdTitle-1);
  UpdatePlayerBounds; // apply VideoAspect
end;

procedure TNkVpMainForm.cmFitMovie;
begin
  FitMovie := not FitMovie;
end;

procedure TNkVpMainForm.cmZoom50;
begin
  Zoom(50);
end;

procedure TNkVpMainForm.cmZoom100;
begin
  Zoom(100);
end;

procedure TNkVpMainForm.cmZoom150;
begin
  Zoom(150);
end;

procedure TNkVpMainForm.cmZoom200;
begin
  Zoom(200);
end;

procedure TNkVpMainForm.cmZoom300;
begin
  Zoom(300);
end;

procedure TNkVpMainForm.cmZoom400;
begin
  Zoom(400);
end;

procedure TNkVpMainForm.cmPlaylist; // dummy
begin
end;

procedure TNkVpMainForm.cmFilterMenu; // dummy
begin
end;

procedure TNkVpMainForm.cmTitle; // dummy
begin
end;

procedure TNkVpMainForm.cmAudioStream; // dummy
begin
end;

procedure TNkVpMainForm.cmSubpictureStream; // dummy
begin
end;

procedure TNkVpMainForm.cmShowSubpicture;
var
  n, cur: Cardinal;
  display: Boolean;
begin
  Player.DvdGetSubpicture(n, cur, display);
  Player.DvdShowSubpicture(not display);
end;

procedure TNkVpMainForm.cmAngle; // dummy
begin
end;

//procedure TNkVpMainForm.cmClosedCaption;
//begin
//  Player.SetClosedCaption(not Player.ClosedCaption);
//end;

end.
